# Generated by Django 3.0.3 on 2020-12-28 11:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('insuree', '0004_confirmationtype_education_profession_relation'),
        ('contract', '0004_auto_20201222_1424'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='contractdetails',
            name='policy_holder_insuree',
        ),
        migrations.RemoveField(
            model_name='historicalcontractdetails',
            name='policy_holder_insuree',
        ),
        migrations.AddField(
            model_name='contractdetails',
            name='insuree',
            field=models.ForeignKey(db_column='InsureeID', default=1, on_delete=django.db.models.deletion.DO_NOTHING, to='insuree.Insuree'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='historicalcontractdetails',
            name='insuree',
            field=models.ForeignKey(blank=True, db_column='InsureeID', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='insuree.Insuree'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='amendment',
            field=models.IntegerField(db_column='Amendment', default=0),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='amendment',
            field=models.IntegerField(db_column='Amendment', default=0),
        ),
    ]
