# Generated by Django 3.0.3 on 2020-12-28 12:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contract', '0005_auto_20201228_1151'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='contractcontributionplandetails',
            name='date_valid_from',
        ),
        migrations.RemoveField(
            model_name='contractcontributionplandetails',
            name='date_valid_to',
        ),
        migrations.RemoveField(
            model_name='contractcontributionplandetails',
            name='replacement_uuid',
        ),
        migrations.RemoveField(
            model_name='historicalcontractcontributionplandetails',
            name='date_valid_from',
        ),
        migrations.RemoveField(
            model_name='historicalcontractcontributionplandetails',
            name='date_valid_to',
        ),
        migrations.RemoveField(
            model_name='historicalcontractcontributionplandetails',
            name='replacement_uuid',
        ),
    ]
