# Generated by Django 3.0.3 on 2020-12-22 14:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contribution', '__first__'),
        ('contract', '0003_auto_20201222_1044'),
    ]

    operations = [
        migrations.AddField(
            model_name='contractcontributionplandetails',
            name='contribution',
            field=models.ForeignKey(blank=True, db_column='ContributionId', null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='contribution.Premium'),
        ),
        migrations.AddField(
            model_name='historicalcontractcontributionplandetails',
            name='contribution',
            field=models.ForeignKey(blank=True, db_column='ContributionId', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='contribution.Premium'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='amendment',
            field=models.IntegerField(db_column='Amendment', default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='amendment',
            field=models.IntegerField(db_column='Amendment', default=0),
            preserve_default=False,
        ),
    ]
